@@ -2,24 +2,23 @@
 
   silcnotify.c
 
-  Author: Pekka Riikonen <priikone@poseidon.pspt.fi>
+  Author: Pekka Riikonen <priikone@silcnet.org>
 
-  Copyright (C) 2000 Pekka Riikonen
+  Copyright (C) 2000 - 2007 Pekka Riikonen
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-  
+  the Free Software Foundation; version 2 of the License.
+
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
 */
-/* $Id: silcnotify.c,v 1.12 2002/02/24 11:14:41 priikone Exp $ */
+/* $Id: silcnotify.c,v 1.23.2.1 2007/07/01 13:56:39 priikone Exp $ */
 
-#include "silcincludes.h"
+#include "silc.h"
 #include "silcnotify.h"
 
 /******************************************************************************
@@ -40,37 +39,42 @@ SilcNotifyPayload silc_notify_payload_parse(const unsigned char *payload,
 					    SilcUInt32 payload_len)
 {
   SilcBufferStruct buffer;
-  SilcNotifyPayload new;
+  SilcNotifyPayload newp;
   SilcUInt16 len;
   int ret;
 
   SILC_LOG_DEBUG(("Parsing Notify payload"));
 
   silc_buffer_set(&buffer, (unsigned char *)payload, payload_len);
-  new = silc_calloc(1, sizeof(*new));
+  newp = silc_calloc(1, sizeof(*newp));
+  if (!newp)
+    return NULL;
 
   ret = silc_buffer_unformat(&buffer,
-			     SILC_STR_UI_SHORT(&new->type),
+			     SILC_STR_UI_SHORT(&newp->type),
 			     SILC_STR_UI_SHORT(&len),
-			     SILC_STR_UI_CHAR(&new->argc),
+			     SILC_STR_UI_CHAR(&newp->argc),
 			     SILC_STR_END);
   if (ret == -1)
     goto err;
 
-  if (len > buffer.len)
+  if (len > silc_buffer_len(&buffer))
     goto err;
 
-  if (new->argc) {
+  if (newp->argc) {
     silc_buffer_pull(&buffer, 5);
-    new->args = silc_argument_payload_parse(buffer.data, buffer.len, 
-					    new->argc);
+    newp->args = silc_argument_payload_parse(buffer.data,
+					     silc_buffer_len(&buffer),
+					     newp->argc);
+    if (!newp->args)
+      goto err;
     silc_buffer_push(&buffer, 5);
   }
 
-  return new;
+  return newp;
 
  err:
-  silc_free(new);
+  silc_free(newp);
   return NULL;
 }
 
@@ -78,7 +82,7 @@ SilcNotifyPayload silc_notify_payload_parse(const unsigned char *payload,
    argument payloads will be associated to the notify payload. Variable
    arguments must be {usigned char *, SilcUInt32 (len)}. */
 
-SilcBuffer silc_notify_payload_encode(SilcNotifyType type, SilcUInt32 argc, 
+SilcBuffer silc_notify_payload_encode(SilcNotifyType type, SilcUInt32 argc,
 				      va_list ap)
 {
   SilcBuffer buffer;
@@ -86,29 +90,42 @@ SilcBuffer silc_notify_payload_encode(SilcNotifyType type, SilcUInt32 argc,
   unsigned char **argv;
   SilcUInt32 *argv_lens = NULL, *argv_types = NULL;
   unsigned char *x;
-  SilcUInt32 x_len;
-  int i, k = 0, len = 0;
+  SilcUInt32 x_len, len = 0;
+  int i, k = 0;
 
   if (argc) {
     argv = silc_calloc(argc, sizeof(unsigned char *));
+    if (!argv)
+      return NULL;
     argv_lens = silc_calloc(argc, sizeof(SilcUInt32));
+    if (!argv_lens) {
+      silc_free(argv);
+      return NULL;
+    }
     argv_types = silc_calloc(argc, sizeof(SilcUInt32));
-    
+    if (!argv_types) {
+      silc_free(argv_lens);
+      silc_free(argv);
+      return NULL;
+    }
+
     for (i = 0, k = 0; i < argc; i++) {
       x = va_arg(ap, unsigned char *);
       x_len = va_arg(ap, SilcUInt32);
 
       if (!x || !x_len)
 	continue;
-      
+
       argv[k] = silc_memdup(x, x_len);
+      if (!argv[k])
+	return NULL;
       argv_lens[k] = x_len;
       argv_types[k] = i + 1;
       k++;
     }
 
     args = silc_argument_payload_encode(k, argv, argv_lens, argv_types);
-    len = args->len;
+    len = silc_buffer_len(args);
 
     for (i = 0; i < k; i++)
       silc_free(argv[i]);
@@ -118,8 +135,9 @@ SilcBuffer silc_notify_payload_encode(SilcNotifyType type, SilcUInt32 argc,
   }
 
   len += 5;
-  buffer = silc_buffer_alloc(len);
-  silc_buffer_pull_tail(buffer, SILC_BUFFER_END(buffer));
+  buffer = silc_buffer_alloc_size(len);
+  if (!buffer)
+    return NULL;
   silc_buffer_format(buffer,
 		     SILC_STR_UI_SHORT(type),
 		     SILC_STR_UI_SHORT(len),
@@ -127,11 +145,10 @@ SilcBuffer silc_notify_payload_encode(SilcNotifyType type, SilcUInt32 argc,
 		     SILC_STR_END);
 
   if (k) {
-    silc_buffer_pull(buffer, 5);
     silc_buffer_format(buffer,
-		       SILC_STR_UI_XNSTRING(args->data, args->len),
+		       SILC_STR_OFFSET(5),
+		       SILC_STR_DATA(args->data, silc_buffer_len(args)),
 		       SILC_STR_END);
-    silc_buffer_push(buffer, 5);
     silc_buffer_free(args);
   }
 
@@ -140,29 +157,28 @@ SilcBuffer silc_notify_payload_encode(SilcNotifyType type, SilcUInt32 argc,
 
 /* Same as above but takes argument from the `args' Argument Payload. */
 
-SilcBuffer silc_notify_payload_encode_args(SilcNotifyType type, 
+SilcBuffer silc_notify_payload_encode_args(SilcNotifyType type,
 					   SilcUInt32 argc,
 					   SilcBuffer args)
 {
   SilcBuffer buffer;
-  int len;
+  SilcUInt32 len;
 
-  len = 5 + (args ? args->len : 0);
-  buffer = silc_buffer_alloc(len);
-  silc_buffer_pull_tail(buffer, SILC_BUFFER_END(buffer));
+  len = 5 + (args ? silc_buffer_len(args) : 0);
+  buffer = silc_buffer_alloc_size(len);
+  if (!buffer)
+    return NULL;
   silc_buffer_format(buffer,
 		     SILC_STR_UI_SHORT(type),
 		     SILC_STR_UI_SHORT(len),
 		     SILC_STR_UI_CHAR(argc),
 		     SILC_STR_END);
 
-  if (args) {
-    silc_buffer_pull(buffer, 5);
+  if (args)
     silc_buffer_format(buffer,
-		       SILC_STR_UI_XNSTRING(args->data, args->len),
+		       SILC_STR_OFFSET(5),
+		       SILC_STR_DATA(args->data, silc_buffer_len(args)),
 		       SILC_STR_END);
-    silc_buffer_push(buffer, 5);
-  }
 
   return buffer;
 }
